"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class AcceptCallRequest(TLRequest):
    """TL type: phone.acceptCall"""
    CONSTRUCTOR_ID = 0x3bd2b4a0
    SUBCLASS_OF_ID = 0xd48afe4f

    def __init__(self, peer: 'TypeInputPhoneCall', g_b: bytes, protocol: 'TypePhoneCallProtocol'):
        self.peer = peer
        self.g_b = g_b
        self.protocol = protocol

    def to_dict(self):
        return {
            '_': 'AcceptCallRequest',
            'peer': self.peer,
            'g_b': self.g_b,
            'protocol': self.protocol,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(TLObject.serialize_bytes(self.g_b))
        buf.write(bytes(self.protocol))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_g_b = reader.tgread_bytes()
        _args['g_b'] = _val_g_b
        _val_protocol = reader.tgread_object()
        _args['protocol'] = _val_protocol
        return cls(**_args)


class CheckGroupCallRequest(TLRequest):
    """TL type: phone.checkGroupCall"""
    CONSTRUCTOR_ID = 0xb59cf977
    SUBCLASS_OF_ID = 0x5026710f

    def __init__(self, call: 'TypeInputGroupCall', sources: List[int]):
        self.call = call
        self.sources = sources

    def to_dict(self):
        return {
            '_': 'CheckGroupCallRequest',
            'call': self.call,
            'sources': self.sources,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.sources)))
        for item in self.sources:
            buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        reader.read_int(signed=False)  # skip vector id
        _count_sources = reader.read_int()
        _list_sources = []
        for _ in range(_count_sources):
            _item_sources = reader.read_int()
            _list_sources.append(_item_sources)
        _args['sources'] = _list_sources
        return cls(**_args)


class ConfirmCallRequest(TLRequest):
    """TL type: phone.confirmCall"""
    CONSTRUCTOR_ID = 0x2efe1722
    SUBCLASS_OF_ID = 0xd48afe4f

    def __init__(self, peer: 'TypeInputPhoneCall', g_a: bytes, key_fingerprint: int, protocol: 'TypePhoneCallProtocol'):
        self.peer = peer
        self.g_a = g_a
        self.key_fingerprint = key_fingerprint
        self.protocol = protocol

    def to_dict(self):
        return {
            '_': 'ConfirmCallRequest',
            'peer': self.peer,
            'g_a': self.g_a,
            'key_fingerprint': self.key_fingerprint,
            'protocol': self.protocol,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(TLObject.serialize_bytes(self.g_a))
        buf.write(struct.pack('<q', self.key_fingerprint))
        buf.write(bytes(self.protocol))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_g_a = reader.tgread_bytes()
        _args['g_a'] = _val_g_a
        _val_key_fingerprint = reader.read_long()
        _args['key_fingerprint'] = _val_key_fingerprint
        _val_protocol = reader.tgread_object()
        _args['protocol'] = _val_protocol
        return cls(**_args)


class CreateConferenceCallRequest(TLRequest):
    """TL type: phone.createConferenceCall"""
    CONSTRUCTOR_ID = 0x7d0444bb
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, muted: Optional[bool] = None, video_stopped: Optional[bool] = None, join: Optional[bool] = None, random_id: int = None, public_key: Optional[int] = None, block: Optional[bytes] = None, params: Optional['TypeDataJSON'] = None):
        self.muted = muted
        self.video_stopped = video_stopped
        self.join = join
        self.random_id = random_id
        self.public_key = public_key
        self.block = block
        self.params = params

    def to_dict(self):
        return {
            '_': 'CreateConferenceCallRequest',
            'muted': self.muted,
            'video_stopped': self.video_stopped,
            'join': self.join,
            'random_id': self.random_id,
            'public_key': self.public_key,
            'block': self.block,
            'params': self.params,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.muted:
            flags |= (1 << 0)
        if self.video_stopped:
            flags |= (1 << 2)
        if self.params is not None:
            flags |= (1 << 3)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.random_id))
        if self.public_key is not None:
            buf.write(self.public_key.to_bytes(32, 'little'))
        if self.block is not None:
            buf.write(TLObject.serialize_bytes(self.block))
        if self.params is not None:
            buf.write(bytes(self.params))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['muted'] = bool(flags & (1 << 0))
        _args['video_stopped'] = bool(flags & (1 << 2))
        _args['join'] = bool(flags & (1 << 3))
        _val_random_id = reader.read_int()
        _args['random_id'] = _val_random_id
        if flags & (1 << 3):
            _val_public_key = reader.read_large_int(256)
            _args['public_key'] = _val_public_key
        else:
            _args['public_key'] = None
        if flags & (1 << 3):
            _val_block = reader.tgread_bytes()
            _args['block'] = _val_block
        else:
            _args['block'] = None
        if flags & (1 << 3):
            _val_params = reader.tgread_object()
            _args['params'] = _val_params
        else:
            _args['params'] = None
        return cls(**_args)


class CreateGroupCallRequest(TLRequest):
    """TL type: phone.createGroupCall"""
    CONSTRUCTOR_ID = 0x48cdc6d8
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, peer: 'TypeInputPeer', rtmp_stream: Optional[bool] = None, random_id: int = None, title: Optional[str] = None, schedule_date: Optional[datetime] = None):
        self.peer = peer
        self.rtmp_stream = rtmp_stream
        self.random_id = random_id
        self.title = title
        self.schedule_date = schedule_date

    def to_dict(self):
        return {
            '_': 'CreateGroupCallRequest',
            'peer': self.peer,
            'rtmp_stream': self.rtmp_stream,
            'random_id': self.random_id,
            'title': self.title,
            'schedule_date': self.schedule_date,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.rtmp_stream:
            flags |= (1 << 2)
        if self.title is not None:
            flags |= (1 << 0)
        if self.schedule_date is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        buf.write(struct.pack('<i', self.random_id))
        if self.title is not None:
            buf.write(TLObject.serialize_bytes(self.title))
        if self.schedule_date is not None:
            buf.write(struct.pack('<i', self.schedule_date))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['rtmp_stream'] = bool(flags & (1 << 2))
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_random_id = reader.read_int()
        _args['random_id'] = _val_random_id
        if flags & (1 << 0):
            _val_title = reader.tgread_string()
            _args['title'] = _val_title
        else:
            _args['title'] = None
        if flags & (1 << 1):
            _val_schedule_date = reader.read_int()
            _args['schedule_date'] = _val_schedule_date
        else:
            _args['schedule_date'] = None
        return cls(**_args)


class DeclineConferenceCallInviteRequest(TLRequest):
    """TL type: phone.declineConferenceCallInvite"""
    CONSTRUCTOR_ID = 0x3c479971
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, msg_id: int):
        self.msg_id = msg_id

    def to_dict(self):
        return {
            '_': 'DeclineConferenceCallInviteRequest',
            'msg_id': self.msg_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.msg_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_msg_id = reader.read_int()
        _args['msg_id'] = _val_msg_id
        return cls(**_args)


class DeleteConferenceCallParticipantsRequest(TLRequest):
    """TL type: phone.deleteConferenceCallParticipants"""
    CONSTRUCTOR_ID = 0x8ca60525
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', ids: List[int], block: bytes, only_left: Optional[bool] = None, kick: Optional[bool] = None):
        self.call = call
        self.ids = ids
        self.block = block
        self.only_left = only_left
        self.kick = kick

    def to_dict(self):
        return {
            '_': 'DeleteConferenceCallParticipantsRequest',
            'call': self.call,
            'ids': self.ids,
            'block': self.block,
            'only_left': self.only_left,
            'kick': self.kick,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.only_left:
            flags |= (1 << 0)
        if self.kick:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.ids)))
        for item in self.ids:
            buf.write(struct.pack('<q', self.item))
        buf.write(TLObject.serialize_bytes(self.block))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['only_left'] = bool(flags & (1 << 0))
        _args['kick'] = bool(flags & (1 << 1))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        reader.read_int(signed=False)  # skip vector id
        _count_ids = reader.read_int()
        _list_ids = []
        for _ in range(_count_ids):
            _item_ids = reader.read_long()
            _list_ids.append(_item_ids)
        _args['ids'] = _list_ids
        _val_block = reader.tgread_bytes()
        _args['block'] = _val_block
        return cls(**_args)


class DeleteGroupCallMessagesRequest(TLRequest):
    """TL type: phone.deleteGroupCallMessages"""
    CONSTRUCTOR_ID = 0xf64f54f7
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', messages: List[int], report_spam: Optional[bool] = None):
        self.call = call
        self.messages = messages
        self.report_spam = report_spam

    def to_dict(self):
        return {
            '_': 'DeleteGroupCallMessagesRequest',
            'call': self.call,
            'messages': self.messages,
            'report_spam': self.report_spam,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.report_spam:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.messages)))
        for item in self.messages:
            buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['report_spam'] = bool(flags & (1 << 0))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        reader.read_int(signed=False)  # skip vector id
        _count_messages = reader.read_int()
        _list_messages = []
        for _ in range(_count_messages):
            _item_messages = reader.read_int()
            _list_messages.append(_item_messages)
        _args['messages'] = _list_messages
        return cls(**_args)


class DeleteGroupCallParticipantMessagesRequest(TLRequest):
    """TL type: phone.deleteGroupCallParticipantMessages"""
    CONSTRUCTOR_ID = 0x1dbfeca0
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', participant: 'TypeInputPeer', report_spam: Optional[bool] = None):
        self.call = call
        self.participant = participant
        self.report_spam = report_spam

    def to_dict(self):
        return {
            '_': 'DeleteGroupCallParticipantMessagesRequest',
            'call': self.call,
            'participant': self.participant,
            'report_spam': self.report_spam,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.report_spam:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        buf.write(bytes(self.participant))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['report_spam'] = bool(flags & (1 << 0))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_participant = reader.tgread_object()
        _args['participant'] = _val_participant
        return cls(**_args)


class DiscardCallRequest(TLRequest):
    """TL type: phone.discardCall"""
    CONSTRUCTOR_ID = 0xb2cbc1c0
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, peer: 'TypeInputPhoneCall', duration: int, reason: 'TypePhoneCallDiscardReason', connection_id: int, video: Optional[bool] = None):
        self.peer = peer
        self.duration = duration
        self.reason = reason
        self.connection_id = connection_id
        self.video = video

    def to_dict(self):
        return {
            '_': 'DiscardCallRequest',
            'peer': self.peer,
            'duration': self.duration,
            'reason': self.reason,
            'connection_id': self.connection_id,
            'video': self.video,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.video:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        buf.write(struct.pack('<i', self.duration))
        buf.write(bytes(self.reason))
        buf.write(struct.pack('<q', self.connection_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['video'] = bool(flags & (1 << 0))
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_duration = reader.read_int()
        _args['duration'] = _val_duration
        _val_reason = reader.tgread_object()
        _args['reason'] = _val_reason
        _val_connection_id = reader.read_long()
        _args['connection_id'] = _val_connection_id
        return cls(**_args)


class DiscardGroupCallRequest(TLRequest):
    """TL type: phone.discardGroupCall"""
    CONSTRUCTOR_ID = 0x7a777135
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall'):
        self.call = call

    def to_dict(self):
        return {
            '_': 'DiscardGroupCallRequest',
            'call': self.call,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        return cls(**_args)


class EditGroupCallParticipantRequest(TLRequest):
    """TL type: phone.editGroupCallParticipant"""
    CONSTRUCTOR_ID = 0xa5273abf
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', participant: 'TypeInputPeer', muted: Optional[bool] = None, volume: Optional[int] = None, raise_hand: Optional[bool] = None, video_stopped: Optional[bool] = None, video_paused: Optional[bool] = None, presentation_paused: Optional[bool] = None):
        self.call = call
        self.participant = participant
        self.muted = muted
        self.volume = volume
        self.raise_hand = raise_hand
        self.video_stopped = video_stopped
        self.video_paused = video_paused
        self.presentation_paused = presentation_paused

    def to_dict(self):
        return {
            '_': 'EditGroupCallParticipantRequest',
            'call': self.call,
            'participant': self.participant,
            'muted': self.muted,
            'volume': self.volume,
            'raise_hand': self.raise_hand,
            'video_stopped': self.video_stopped,
            'video_paused': self.video_paused,
            'presentation_paused': self.presentation_paused,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.muted is not None:
            flags |= (1 << 0)
        if self.volume is not None:
            flags |= (1 << 1)
        if self.raise_hand is not None:
            flags |= (1 << 2)
        if self.video_stopped is not None:
            flags |= (1 << 3)
        if self.video_paused is not None:
            flags |= (1 << 4)
        if self.presentation_paused is not None:
            flags |= (1 << 5)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        buf.write(bytes(self.participant))
        if self.muted is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.muted else 0xbc799737))
        if self.volume is not None:
            buf.write(struct.pack('<i', self.volume))
        if self.raise_hand is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.raise_hand else 0xbc799737))
        if self.video_stopped is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.video_stopped else 0xbc799737))
        if self.video_paused is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.video_paused else 0xbc799737))
        if self.presentation_paused is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.presentation_paused else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_participant = reader.tgread_object()
        _args['participant'] = _val_participant
        if flags & (1 << 0):
            _val_muted = reader.tgread_bool()
            _args['muted'] = _val_muted
        else:
            _args['muted'] = None
        if flags & (1 << 1):
            _val_volume = reader.read_int()
            _args['volume'] = _val_volume
        else:
            _args['volume'] = None
        if flags & (1 << 2):
            _val_raise_hand = reader.tgread_bool()
            _args['raise_hand'] = _val_raise_hand
        else:
            _args['raise_hand'] = None
        if flags & (1 << 3):
            _val_video_stopped = reader.tgread_bool()
            _args['video_stopped'] = _val_video_stopped
        else:
            _args['video_stopped'] = None
        if flags & (1 << 4):
            _val_video_paused = reader.tgread_bool()
            _args['video_paused'] = _val_video_paused
        else:
            _args['video_paused'] = None
        if flags & (1 << 5):
            _val_presentation_paused = reader.tgread_bool()
            _args['presentation_paused'] = _val_presentation_paused
        else:
            _args['presentation_paused'] = None
        return cls(**_args)


class EditGroupCallTitleRequest(TLRequest):
    """TL type: phone.editGroupCallTitle"""
    CONSTRUCTOR_ID = 0x1ca6ac0a
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', title: str):
        self.call = call
        self.title = title

    def to_dict(self):
        return {
            '_': 'EditGroupCallTitleRequest',
            'call': self.call,
            'title': self.title,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(TLObject.serialize_bytes(self.title))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        return cls(**_args)


class ExportGroupCallInviteRequest(TLRequest):
    """TL type: phone.exportGroupCallInvite"""
    CONSTRUCTOR_ID = 0xe6aa647f
    SUBCLASS_OF_ID = 0x3b3bfe8f

    def __init__(self, call: 'TypeInputGroupCall', can_self_unmute: Optional[bool] = None):
        self.call = call
        self.can_self_unmute = can_self_unmute

    def to_dict(self):
        return {
            '_': 'ExportGroupCallInviteRequest',
            'call': self.call,
            'can_self_unmute': self.can_self_unmute,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.can_self_unmute:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['can_self_unmute'] = bool(flags & (1 << 0))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        return cls(**_args)


class GetCallConfigRequest(TLRequest):
    """TL type: phone.getCallConfig"""
    CONSTRUCTOR_ID = 0x55451fa9
    SUBCLASS_OF_ID = 0xad0352e8

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetCallConfigRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetGroupCallRequest(TLRequest):
    """TL type: phone.getGroupCall"""
    CONSTRUCTOR_ID = 0x041845db
    SUBCLASS_OF_ID = 0x304116be

    def __init__(self, call: 'TypeInputGroupCall', limit: int):
        self.call = call
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetGroupCallRequest',
            'call': self.call,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetGroupCallChainBlocksRequest(TLRequest):
    """TL type: phone.getGroupCallChainBlocks"""
    CONSTRUCTOR_ID = 0xee9f88a6
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', sub_chain_id: int, offset: int, limit: int):
        self.call = call
        self.sub_chain_id = sub_chain_id
        self.offset = offset
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetGroupCallChainBlocksRequest',
            'call': self.call,
            'sub_chain_id': self.sub_chain_id,
            'offset': self.offset,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', self.sub_chain_id))
        buf.write(struct.pack('<i', self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_sub_chain_id = reader.read_int()
        _args['sub_chain_id'] = _val_sub_chain_id
        _val_offset = reader.read_int()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetGroupCallJoinAsRequest(TLRequest):
    """TL type: phone.getGroupCallJoinAs"""
    CONSTRUCTOR_ID = 0xef7c213a
    SUBCLASS_OF_ID = 0xb4b770fb

    def __init__(self, peer: 'TypeInputPeer'):
        self.peer = peer

    def to_dict(self):
        return {
            '_': 'GetGroupCallJoinAsRequest',
            'peer': self.peer,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        return cls(**_args)


class GetGroupCallStarsRequest(TLRequest):
    """TL type: phone.getGroupCallStars"""
    CONSTRUCTOR_ID = 0x6f636302
    SUBCLASS_OF_ID = 0xb447801a

    def __init__(self, call: 'TypeInputGroupCall'):
        self.call = call

    def to_dict(self):
        return {
            '_': 'GetGroupCallStarsRequest',
            'call': self.call,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        return cls(**_args)


class GetGroupCallStreamChannelsRequest(TLRequest):
    """TL type: phone.getGroupCallStreamChannels"""
    CONSTRUCTOR_ID = 0x1ab21940
    SUBCLASS_OF_ID = 0x9157c5e4

    def __init__(self, call: 'TypeInputGroupCall'):
        self.call = call

    def to_dict(self):
        return {
            '_': 'GetGroupCallStreamChannelsRequest',
            'call': self.call,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        return cls(**_args)


class GetGroupCallStreamRtmpUrlRequest(TLRequest):
    """TL type: phone.getGroupCallStreamRtmpUrl"""
    CONSTRUCTOR_ID = 0x5af4c73a
    SUBCLASS_OF_ID = 0xd1f515cb

    def __init__(self, peer: 'TypeInputPeer', revoke: bool, live_story: Optional[bool] = None):
        self.peer = peer
        self.revoke = revoke
        self.live_story = live_story

    def to_dict(self):
        return {
            '_': 'GetGroupCallStreamRtmpUrlRequest',
            'peer': self.peer,
            'revoke': self.revoke,
            'live_story': self.live_story,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.live_story:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        buf.write(struct.pack('<I', 0x997275b5 if self.revoke else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['live_story'] = bool(flags & (1 << 0))
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_revoke = reader.tgread_bool()
        _args['revoke'] = _val_revoke
        return cls(**_args)


class GetGroupParticipantsRequest(TLRequest):
    """TL type: phone.getGroupParticipants"""
    CONSTRUCTOR_ID = 0xc558d8ab
    SUBCLASS_OF_ID = 0x72d304f4

    def __init__(self, call: 'TypeInputGroupCall', ids: List['TypeInputPeer'], sources: List[int], offset: str, limit: int):
        self.call = call
        self.ids = ids
        self.sources = sources
        self.offset = offset
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetGroupParticipantsRequest',
            'call': self.call,
            'ids': self.ids,
            'sources': self.sources,
            'offset': self.offset,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.ids)))
        for item in self.ids:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.sources)))
        for item in self.sources:
            buf.write(struct.pack('<i', self.item))
        buf.write(TLObject.serialize_bytes(self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        reader.read_int(signed=False)  # skip vector id
        _count_ids = reader.read_int()
        _list_ids = []
        for _ in range(_count_ids):
            _item_ids = reader.tgread_object()
            _list_ids.append(_item_ids)
        _args['ids'] = _list_ids
        reader.read_int(signed=False)  # skip vector id
        _count_sources = reader.read_int()
        _list_sources = []
        for _ in range(_count_sources):
            _item_sources = reader.read_int()
            _list_sources.append(_item_sources)
        _args['sources'] = _list_sources
        _val_offset = reader.tgread_string()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class InviteConferenceCallParticipantRequest(TLRequest):
    """TL type: phone.inviteConferenceCallParticipant"""
    CONSTRUCTOR_ID = 0xbcf22685
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', user_id: 'TypeInputUser', video: Optional[bool] = None):
        self.call = call
        self.user_id = user_id
        self.video = video

    def to_dict(self):
        return {
            '_': 'InviteConferenceCallParticipantRequest',
            'call': self.call,
            'user_id': self.user_id,
            'video': self.video,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.video:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        buf.write(bytes(self.user_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['video'] = bool(flags & (1 << 0))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        return cls(**_args)


class InviteToGroupCallRequest(TLRequest):
    """TL type: phone.inviteToGroupCall"""
    CONSTRUCTOR_ID = 0x7b393160
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', users: List['TypeInputUser']):
        self.call = call
        self.users = users

    def to_dict(self):
        return {
            '_': 'InviteToGroupCallRequest',
            'call': self.call,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class JoinGroupCallRequest(TLRequest):
    """TL type: phone.joinGroupCall"""
    CONSTRUCTOR_ID = 0x8fb53057
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', join_as: 'TypeInputPeer', params: 'TypeDataJSON', muted: Optional[bool] = None, video_stopped: Optional[bool] = None, invite_hash: Optional[str] = None, public_key: Optional[int] = None, block: Optional[bytes] = None):
        self.call = call
        self.join_as = join_as
        self.params = params
        self.muted = muted
        self.video_stopped = video_stopped
        self.invite_hash = invite_hash
        self.public_key = public_key
        self.block = block

    def to_dict(self):
        return {
            '_': 'JoinGroupCallRequest',
            'call': self.call,
            'join_as': self.join_as,
            'params': self.params,
            'muted': self.muted,
            'video_stopped': self.video_stopped,
            'invite_hash': self.invite_hash,
            'public_key': self.public_key,
            'block': self.block,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.muted:
            flags |= (1 << 0)
        if self.video_stopped:
            flags |= (1 << 2)
        if self.invite_hash is not None:
            flags |= (1 << 1)
        if self.block is not None:
            flags |= (1 << 3)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        buf.write(bytes(self.join_as))
        buf.write(bytes(self.params))
        if self.invite_hash is not None:
            buf.write(TLObject.serialize_bytes(self.invite_hash))
        if self.public_key is not None:
            buf.write(self.public_key.to_bytes(32, 'little'))
        if self.block is not None:
            buf.write(TLObject.serialize_bytes(self.block))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['muted'] = bool(flags & (1 << 0))
        _args['video_stopped'] = bool(flags & (1 << 2))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_join_as = reader.tgread_object()
        _args['join_as'] = _val_join_as
        if flags & (1 << 1):
            _val_invite_hash = reader.tgread_string()
            _args['invite_hash'] = _val_invite_hash
        else:
            _args['invite_hash'] = None
        if flags & (1 << 3):
            _val_public_key = reader.read_large_int(256)
            _args['public_key'] = _val_public_key
        else:
            _args['public_key'] = None
        if flags & (1 << 3):
            _val_block = reader.tgread_bytes()
            _args['block'] = _val_block
        else:
            _args['block'] = None
        _val_params = reader.tgread_object()
        _args['params'] = _val_params
        return cls(**_args)


class JoinGroupCallPresentationRequest(TLRequest):
    """TL type: phone.joinGroupCallPresentation"""
    CONSTRUCTOR_ID = 0xcbea6bc4
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', params: 'TypeDataJSON'):
        self.call = call
        self.params = params

    def to_dict(self):
        return {
            '_': 'JoinGroupCallPresentationRequest',
            'call': self.call,
            'params': self.params,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(bytes(self.params))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_params = reader.tgread_object()
        _args['params'] = _val_params
        return cls(**_args)


class LeaveGroupCallRequest(TLRequest):
    """TL type: phone.leaveGroupCall"""
    CONSTRUCTOR_ID = 0x500377f9
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', source: int):
        self.call = call
        self.source = source

    def to_dict(self):
        return {
            '_': 'LeaveGroupCallRequest',
            'call': self.call,
            'source': self.source,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<i', self.source))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_source = reader.read_int()
        _args['source'] = _val_source
        return cls(**_args)


class LeaveGroupCallPresentationRequest(TLRequest):
    """TL type: phone.leaveGroupCallPresentation"""
    CONSTRUCTOR_ID = 0x1c50d144
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall'):
        self.call = call

    def to_dict(self):
        return {
            '_': 'LeaveGroupCallPresentationRequest',
            'call': self.call,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        return cls(**_args)


class ReceivedCallRequest(TLRequest):
    """TL type: phone.receivedCall"""
    CONSTRUCTOR_ID = 0x17d54f61
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPhoneCall'):
        self.peer = peer

    def to_dict(self):
        return {
            '_': 'ReceivedCallRequest',
            'peer': self.peer,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        return cls(**_args)


class RequestCallRequest(TLRequest):
    """TL type: phone.requestCall"""
    CONSTRUCTOR_ID = 0x42ff96ed
    SUBCLASS_OF_ID = 0xd48afe4f

    def __init__(self, user_id: 'TypeInputUser', g_a_hash: bytes, protocol: 'TypePhoneCallProtocol', video: Optional[bool] = None, random_id: int = None):
        self.user_id = user_id
        self.g_a_hash = g_a_hash
        self.protocol = protocol
        self.video = video
        self.random_id = random_id

    def to_dict(self):
        return {
            '_': 'RequestCallRequest',
            'user_id': self.user_id,
            'g_a_hash': self.g_a_hash,
            'protocol': self.protocol,
            'video': self.video,
            'random_id': self.random_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.video:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.user_id))
        buf.write(TLObject.serialize_bytes(self.g_a_hash))
        buf.write(bytes(self.protocol))
        buf.write(struct.pack('<i', self.random_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['video'] = bool(flags & (1 << 0))
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_random_id = reader.read_int()
        _args['random_id'] = _val_random_id
        _val_g_a_hash = reader.tgread_bytes()
        _args['g_a_hash'] = _val_g_a_hash
        _val_protocol = reader.tgread_object()
        _args['protocol'] = _val_protocol
        return cls(**_args)


class SaveCallDebugRequest(TLRequest):
    """TL type: phone.saveCallDebug"""
    CONSTRUCTOR_ID = 0x277add7e
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPhoneCall', debug: 'TypeDataJSON'):
        self.peer = peer
        self.debug = debug

    def to_dict(self):
        return {
            '_': 'SaveCallDebugRequest',
            'peer': self.peer,
            'debug': self.debug,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(bytes(self.debug))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_debug = reader.tgread_object()
        _args['debug'] = _val_debug
        return cls(**_args)


class SaveCallLogRequest(TLRequest):
    """TL type: phone.saveCallLog"""
    CONSTRUCTOR_ID = 0x41248786
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPhoneCall', file: 'TypeInputFile'):
        self.peer = peer
        self.file = file

    def to_dict(self):
        return {
            '_': 'SaveCallLogRequest',
            'peer': self.peer,
            'file': self.file,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(bytes(self.file))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_file = reader.tgread_object()
        _args['file'] = _val_file
        return cls(**_args)


class SaveDefaultGroupCallJoinAsRequest(TLRequest):
    """TL type: phone.saveDefaultGroupCallJoinAs"""
    CONSTRUCTOR_ID = 0x575e1f8c
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPeer', join_as: 'TypeInputPeer'):
        self.peer = peer
        self.join_as = join_as

    def to_dict(self):
        return {
            '_': 'SaveDefaultGroupCallJoinAsRequest',
            'peer': self.peer,
            'join_as': self.join_as,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(bytes(self.join_as))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_join_as = reader.tgread_object()
        _args['join_as'] = _val_join_as
        return cls(**_args)


class SaveDefaultSendAsRequest(TLRequest):
    """TL type: phone.saveDefaultSendAs"""
    CONSTRUCTOR_ID = 0x4167add1
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, call: 'TypeInputGroupCall', send_as: 'TypeInputPeer'):
        self.call = call
        self.send_as = send_as

    def to_dict(self):
        return {
            '_': 'SaveDefaultSendAsRequest',
            'call': self.call,
            'send_as': self.send_as,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(bytes(self.send_as))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_send_as = reader.tgread_object()
        _args['send_as'] = _val_send_as
        return cls(**_args)


class SendConferenceCallBroadcastRequest(TLRequest):
    """TL type: phone.sendConferenceCallBroadcast"""
    CONSTRUCTOR_ID = 0xc6701900
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', block: bytes):
        self.call = call
        self.block = block

    def to_dict(self):
        return {
            '_': 'SendConferenceCallBroadcastRequest',
            'call': self.call,
            'block': self.block,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(TLObject.serialize_bytes(self.block))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_block = reader.tgread_bytes()
        _args['block'] = _val_block
        return cls(**_args)


class SendGroupCallEncryptedMessageRequest(TLRequest):
    """TL type: phone.sendGroupCallEncryptedMessage"""
    CONSTRUCTOR_ID = 0xe5afa56d
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, call: 'TypeInputGroupCall', encrypted_message: bytes):
        self.call = call
        self.encrypted_message = encrypted_message

    def to_dict(self):
        return {
            '_': 'SendGroupCallEncryptedMessageRequest',
            'call': self.call,
            'encrypted_message': self.encrypted_message,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(TLObject.serialize_bytes(self.encrypted_message))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_encrypted_message = reader.tgread_bytes()
        _args['encrypted_message'] = _val_encrypted_message
        return cls(**_args)


class SendGroupCallMessageRequest(TLRequest):
    """TL type: phone.sendGroupCallMessage"""
    CONSTRUCTOR_ID = 0xb1d11410
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', message: 'TypeTextWithEntities', random_id: int = None, allow_paid_stars: Optional[int] = None, send_as: Optional['TypeInputPeer'] = None):
        self.call = call
        self.message = message
        self.random_id = random_id
        self.allow_paid_stars = allow_paid_stars
        self.send_as = send_as

    def to_dict(self):
        return {
            '_': 'SendGroupCallMessageRequest',
            'call': self.call,
            'message': self.message,
            'random_id': self.random_id,
            'allow_paid_stars': self.allow_paid_stars,
            'send_as': self.send_as,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.allow_paid_stars is not None:
            flags |= (1 << 0)
        if self.send_as is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        buf.write(bytes(self.message))
        buf.write(struct.pack('<q', self.random_id))
        if self.allow_paid_stars is not None:
            buf.write(struct.pack('<q', self.allow_paid_stars))
        if self.send_as is not None:
            buf.write(bytes(self.send_as))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_random_id = reader.read_long()
        _args['random_id'] = _val_random_id
        _val_message = reader.tgread_object()
        _args['message'] = _val_message
        if flags & (1 << 0):
            _val_allow_paid_stars = reader.read_long()
            _args['allow_paid_stars'] = _val_allow_paid_stars
        else:
            _args['allow_paid_stars'] = None
        if flags & (1 << 1):
            _val_send_as = reader.tgread_object()
            _args['send_as'] = _val_send_as
        else:
            _args['send_as'] = None
        return cls(**_args)


class SendSignalingDataRequest(TLRequest):
    """TL type: phone.sendSignalingData"""
    CONSTRUCTOR_ID = 0xff7a9383
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPhoneCall', data: bytes):
        self.peer = peer
        self.data = data

    def to_dict(self):
        return {
            '_': 'SendSignalingDataRequest',
            'peer': self.peer,
            'data': self.data,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(TLObject.serialize_bytes(self.data))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_data = reader.tgread_bytes()
        _args['data'] = _val_data
        return cls(**_args)


class SetCallRatingRequest(TLRequest):
    """TL type: phone.setCallRating"""
    CONSTRUCTOR_ID = 0x59ead627
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, peer: 'TypeInputPhoneCall', rating: int, comment: str, user_initiative: Optional[bool] = None):
        self.peer = peer
        self.rating = rating
        self.comment = comment
        self.user_initiative = user_initiative

    def to_dict(self):
        return {
            '_': 'SetCallRatingRequest',
            'peer': self.peer,
            'rating': self.rating,
            'comment': self.comment,
            'user_initiative': self.user_initiative,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.user_initiative:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        buf.write(struct.pack('<i', self.rating))
        buf.write(TLObject.serialize_bytes(self.comment))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['user_initiative'] = bool(flags & (1 << 0))
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_rating = reader.read_int()
        _args['rating'] = _val_rating
        _val_comment = reader.tgread_string()
        _args['comment'] = _val_comment
        return cls(**_args)


class StartScheduledGroupCallRequest(TLRequest):
    """TL type: phone.startScheduledGroupCall"""
    CONSTRUCTOR_ID = 0x5680e342
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall'):
        self.call = call

    def to_dict(self):
        return {
            '_': 'StartScheduledGroupCallRequest',
            'call': self.call,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        return cls(**_args)


class ToggleGroupCallRecordRequest(TLRequest):
    """TL type: phone.toggleGroupCallRecord"""
    CONSTRUCTOR_ID = 0xf128c708
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', start: Optional[bool] = None, video: Optional[bool] = None, title: Optional[str] = None, video_portrait: Optional[bool] = None):
        self.call = call
        self.start = start
        self.video = video
        self.title = title
        self.video_portrait = video_portrait

    def to_dict(self):
        return {
            '_': 'ToggleGroupCallRecordRequest',
            'call': self.call,
            'start': self.start,
            'video': self.video,
            'title': self.title,
            'video_portrait': self.video_portrait,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.start:
            flags |= (1 << 0)
        if self.video_portrait is not None:
            flags |= (1 << 2)
        if self.title is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        if self.title is not None:
            buf.write(TLObject.serialize_bytes(self.title))
        if self.video_portrait is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.video_portrait else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['start'] = bool(flags & (1 << 0))
        _args['video'] = bool(flags & (1 << 2))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        if flags & (1 << 1):
            _val_title = reader.tgread_string()
            _args['title'] = _val_title
        else:
            _args['title'] = None
        if flags & (1 << 2):
            _val_video_portrait = reader.tgread_bool()
            _args['video_portrait'] = _val_video_portrait
        else:
            _args['video_portrait'] = None
        return cls(**_args)


class ToggleGroupCallSettingsRequest(TLRequest):
    """TL type: phone.toggleGroupCallSettings"""
    CONSTRUCTOR_ID = 0x974392f2
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', reset_invite_hash: Optional[bool] = None, join_muted: Optional[bool] = None, messages_enabled: Optional[bool] = None, send_paid_messages_stars: Optional[int] = None):
        self.call = call
        self.reset_invite_hash = reset_invite_hash
        self.join_muted = join_muted
        self.messages_enabled = messages_enabled
        self.send_paid_messages_stars = send_paid_messages_stars

    def to_dict(self):
        return {
            '_': 'ToggleGroupCallSettingsRequest',
            'call': self.call,
            'reset_invite_hash': self.reset_invite_hash,
            'join_muted': self.join_muted,
            'messages_enabled': self.messages_enabled,
            'send_paid_messages_stars': self.send_paid_messages_stars,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.reset_invite_hash:
            flags |= (1 << 1)
        if self.join_muted is not None:
            flags |= (1 << 0)
        if self.messages_enabled is not None:
            flags |= (1 << 2)
        if self.send_paid_messages_stars is not None:
            flags |= (1 << 3)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.call))
        if self.join_muted is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.join_muted else 0xbc799737))
        if self.messages_enabled is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.messages_enabled else 0xbc799737))
        if self.send_paid_messages_stars is not None:
            buf.write(struct.pack('<q', self.send_paid_messages_stars))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['reset_invite_hash'] = bool(flags & (1 << 1))
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        if flags & (1 << 0):
            _val_join_muted = reader.tgread_bool()
            _args['join_muted'] = _val_join_muted
        else:
            _args['join_muted'] = None
        if flags & (1 << 2):
            _val_messages_enabled = reader.tgread_bool()
            _args['messages_enabled'] = _val_messages_enabled
        else:
            _args['messages_enabled'] = None
        if flags & (1 << 3):
            _val_send_paid_messages_stars = reader.read_long()
            _args['send_paid_messages_stars'] = _val_send_paid_messages_stars
        else:
            _args['send_paid_messages_stars'] = None
        return cls(**_args)


class ToggleGroupCallStartSubscriptionRequest(TLRequest):
    """TL type: phone.toggleGroupCallStartSubscription"""
    CONSTRUCTOR_ID = 0x219c34e6
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, call: 'TypeInputGroupCall', subscribed: bool):
        self.call = call
        self.subscribed = subscribed

    def to_dict(self):
        return {
            '_': 'ToggleGroupCallStartSubscriptionRequest',
            'call': self.call,
            'subscribed': self.subscribed,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.call))
        buf.write(struct.pack('<I', 0x997275b5 if self.subscribed else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_call = reader.tgread_object()
        _args['call'] = _val_call
        _val_subscribed = reader.tgread_bool()
        _args['subscribed'] = _val_subscribed
        return cls(**_args)




"""
Utility class to build Python source code with proper indentation.
"""


class SourceBuilder:
    """Builds indented Python source code line by line."""

    def __init__(self, file):
        self.file = file
        self.current_indent = 0
        self._on_newline = True

    def indent(self):
        self.current_indent += 1

    def dedent(self):
        self.current_indent = max(0, self.current_indent - 1)

    def end_block(self):
        self.dedent()

    def writeln(self, line='', *args, **kwargs):
        if line:
            formatted = line.format(*args, **kwargs) if (args or kwargs) else line
            prefix = '    ' * self.current_indent
            self.file.write(f'{prefix}{formatted}\n')
        else:
            self.file.write('\n')

    def write(self, text):
        self.file.write(text)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        pass



"""
MTProtoState - manages session encryption, message IDs, and sequence numbers.
Implements MTProto 2.0 as per https://core.telegram.org/mtproto/description
"""
import os
import struct
import time
from collections import deque
from hashlib import sha256

from ..crypto import AES
from ..errors import SecurityError, InvalidBufferError
from ..extensions import BinaryReader
from ..tl.core import TLMessage
from ..tl.core.gzippacked import GzipPacked

MAX_RECENT_MSG_IDS = 500
MSG_TOO_NEW_DELTA = 30
MSG_TOO_OLD_DELTA = 300
MAX_CONSECUTIVE_IGNORED = 10


class MTProtoState:
    """
    Holds all state needed to encrypt/decrypt MTProto 2.0 messages:
    - Authorization key
    - Session ID
    - Message ID counter
    - Sequence number
    - Server salt
    - Recent msg_id deque (for duplicate detection)
    """

    def __init__(self, auth_key, loggers=None):
        self.auth_key = auth_key
        self._log = (loggers or {}).get(__name__)
        self.time_offset = 0
        self.salt = 0

        self.id = self._sequence = self._last_msg_id = None
        self._recent_remote_ids = deque(maxlen=MAX_RECENT_MSG_IDS)
        self._highest_remote_id = 0
        self._ignore_count = 0
        self.reset()

    def reset(self):
        self.id = struct.unpack('q', os.urandom(8))[0]
        self._sequence = 0
        self._last_msg_id = 0
        self._recent_remote_ids.clear()
        self._highest_remote_id = 0
        self._ignore_count = 0

    # ── Key derivation (MTProto 2.0) ──────────────────────────────────────

    @staticmethod
    def _calc_key(auth_key: bytes, msg_key: bytes, client: bool):
        """
        Derive AES key+IV from auth_key and msg_key.
        x=0 for client→server, x=8 for server→client.
        """
        x = 0 if client else 8
        sha256a = sha256(msg_key + auth_key[x:x + 36]).digest()
        sha256b = sha256(auth_key[x + 40:x + 76] + msg_key).digest()

        aes_key = sha256a[:8] + sha256b[8:24] + sha256a[24:32]
        aes_iv = sha256b[:8] + sha256a[8:24] + sha256b[24:32]
        return aes_key, aes_iv

    # ── Message ID ────────────────────────────────────────────────────────

    def _get_new_msg_id(self) -> int:
        now = time.time() + self.time_offset
        ns = int((now - int(now)) * 1e9)
        new_msg_id = (int(now) << 32) | (ns << 2)
        if self._last_msg_id >= new_msg_id:
            new_msg_id = self._last_msg_id + 4
        self._last_msg_id = new_msg_id
        return new_msg_id

    def update_message_id(self, message):
        message.msg_id = self._get_new_msg_id()

    def update_time_offset(self, correct_msg_id: int) -> int:
        bad = self._get_new_msg_id()
        old = self.time_offset
        self.time_offset = (correct_msg_id >> 32) - int(time.time())
        if self.time_offset != old:
            self._last_msg_id = 0
        return self.time_offset

    # ── Seq no ───────────────────────────────────────────────────────────

    def _get_seq_no(self, content_related: bool) -> int:
        if content_related:
            result = self._sequence * 2 + 1
            self._sequence += 1
            return result
        return self._sequence * 2

    # ── Framing ──────────────────────────────────────────────────────────

    def write_data_as_message(self, buffer: bytearray, data: bytes,
                               content_related: bool, *, after_id=None) -> int:
        msg_id = self._get_new_msg_id()
        seq_no = self._get_seq_no(content_related)

        if after_id is None:
            body = GzipPacked.gzip_if_smaller(content_related, data)
        else:
            from ..tl.tlobject import TLRequest
            # InvokeAfterMsg wrapper
            wrapped = (
                struct.pack('<Iqq', 0xcb9f372d, after_id, 0) +
                data  # simplified; full impl uses InvokeAfterMsgRequest
            )
            body = GzipPacked.gzip_if_smaller(content_related, data)

        buffer.extend(struct.pack('<qii', msg_id, seq_no, len(body)))
        buffer.extend(body)
        return msg_id

    # ── Encryption / Decryption ───────────────────────────────────────────

    def encrypt_message_data(self, data: bytes) -> bytes:
        """Encrypt data using current auth key (MTProto 2.0)."""
        data = struct.pack('<qq', self.salt, self.id) + data
        padding = os.urandom(-(len(data) + 12) % 16 + 12)

        # msg_key = middle 16 bytes of SHA256(auth_key[88:120] + plaintext + padding)
        msg_key_large = sha256(
            self.auth_key.key[88:88 + 32] + data + padding
        ).digest()
        msg_key = msg_key_large[8:24]

        aes_key, aes_iv = self._calc_key(self.auth_key.key, msg_key, True)
        key_id = struct.pack('<Q', self.auth_key.key_id)
        return key_id + msg_key + AES.encrypt_ige(data + padding, aes_key, aes_iv)

    def decrypt_message_data(self, body: bytes):
        """Decrypt incoming server message data. Returns TLMessage or None."""
        now = time.time()

        if len(body) < 8:
            raise InvalidBufferError(body)

        key_id = struct.unpack('<Q', body[:8])[0]
        if key_id != self.auth_key.key_id:
            raise SecurityError('Server replied with an invalid auth key')

        msg_key = body[8:24]
        aes_key, aes_iv = self._calc_key(self.auth_key.key, msg_key, False)
        body = AES.decrypt_ige(body[24:], aes_key, aes_iv)

        # Verify SHA256
        our_key = sha256(self.auth_key.key[96:96 + 32] + body)
        if msg_key != our_key.digest()[8:24]:
            raise SecurityError("Received msg_key doesn't match expected")

        reader = BinaryReader(body)
        reader.read_long()  # salt
        if reader.read_long() != self.id:
            raise SecurityError('Server replied with wrong session ID')

        remote_msg_id = reader.read_long()
        if remote_msg_id % 2 != 1:
            raise SecurityError('Server sent an even msg_id')

        if (remote_msg_id <= self._highest_remote_id
                and remote_msg_id in self._recent_remote_ids):
            self._count_ignored()
            return None

        remote_sequence = reader.read_int()
        reader.read_int()  # msg_len (padding ignored)

        obj = reader.tgread_object()

        # Time check (skip for BadServerSalt / BadMsgNotification)
        from ..tl.core.rpcresult import RpcResult
        BAD_IDS = {0xedab447b, 0xa7eff811}  # BadServerSalt, BadMsgNotification
        constructor_id = getattr(obj, 'CONSTRUCTOR_ID', None)
        if constructor_id not in BAD_IDS:
            remote_time = remote_msg_id >> 32
            delta = (now + self.time_offset) - remote_time
            if delta > MSG_TOO_OLD_DELTA or -delta > MSG_TOO_NEW_DELTA:
                self._count_ignored()
                return None

        self._recent_remote_ids.append(remote_msg_id)
        self._highest_remote_id = max(self._highest_remote_id, remote_msg_id)
        self._ignore_count = 0

        # Keep time_offset in sync with the server's clock
        self.time_offset = (remote_msg_id >> 32) - int(time.time())

        return TLMessage(remote_msg_id, remote_sequence, obj)

    def _count_ignored(self):
        self._ignore_count += 1
        if self._ignore_count >= MAX_CONSECUTIVE_IGNORED:
            raise SecurityError('Too many consecutive ignored messages')



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class AddPreviewMediaRequest(TLRequest):
    """TL type: bots.addPreviewMedia"""
    CONSTRUCTOR_ID = 0x17aeb75a
    SUBCLASS_OF_ID = 0x562abc2d

    def __init__(self, bot: 'TypeInputUser', lang_code: str, media: 'TypeInputMedia'):
        self.bot = bot
        self.lang_code = lang_code
        self.media = media

    def to_dict(self):
        return {
            '_': 'AddPreviewMediaRequest',
            'bot': self.bot,
            'lang_code': self.lang_code,
            'media': self.media,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        buf.write(bytes(self.media))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        _val_media = reader.tgread_object()
        _args['media'] = _val_media
        return cls(**_args)


class AllowSendMessageRequest(TLRequest):
    """TL type: bots.allowSendMessage"""
    CONSTRUCTOR_ID = 0xf132e3ef
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, bot: 'TypeInputUser'):
        self.bot = bot

    def to_dict(self):
        return {
            '_': 'AllowSendMessageRequest',
            'bot': self.bot,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        return cls(**_args)


class AnswerWebhookJsonqueryRequest(TLRequest):
    """TL type: bots.answerWebhookJSONQuery"""
    CONSTRUCTOR_ID = 0xe6213f4d
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, query_id: int, data: 'TypeDataJSON'):
        self.query_id = query_id
        self.data = data

    def to_dict(self):
        return {
            '_': 'AnswerWebhookJsonqueryRequest',
            'query_id': self.query_id,
            'data': self.data,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.query_id))
        buf.write(bytes(self.data))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_query_id = reader.read_long()
        _args['query_id'] = _val_query_id
        _val_data = reader.tgread_object()
        _args['data'] = _val_data
        return cls(**_args)


class CanSendMessageRequest(TLRequest):
    """TL type: bots.canSendMessage"""
    CONSTRUCTOR_ID = 0x1359f4e6
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, bot: 'TypeInputUser'):
        self.bot = bot

    def to_dict(self):
        return {
            '_': 'CanSendMessageRequest',
            'bot': self.bot,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        return cls(**_args)


class CheckDownloadFileParamsRequest(TLRequest):
    """TL type: bots.checkDownloadFileParams"""
    CONSTRUCTOR_ID = 0x50077589
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, bot: 'TypeInputUser', file_name: str, url: str):
        self.bot = bot
        self.file_name = file_name
        self.url = url

    def to_dict(self):
        return {
            '_': 'CheckDownloadFileParamsRequest',
            'bot': self.bot,
            'file_name': self.file_name,
            'url': self.url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.file_name))
        buf.write(TLObject.serialize_bytes(self.url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_file_name = reader.tgread_string()
        _args['file_name'] = _val_file_name
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        return cls(**_args)


class DeletePreviewMediaRequest(TLRequest):
    """TL type: bots.deletePreviewMedia"""
    CONSTRUCTOR_ID = 0x2d0135b3
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, bot: 'TypeInputUser', lang_code: str, media: List['TypeInputMedia']):
        self.bot = bot
        self.lang_code = lang_code
        self.media = media

    def to_dict(self):
        return {
            '_': 'DeletePreviewMediaRequest',
            'bot': self.bot,
            'lang_code': self.lang_code,
            'media': self.media,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.media)))
        for item in self.media:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        reader.read_int(signed=False)  # skip vector id
        _count_media = reader.read_int()
        _list_media = []
        for _ in range(_count_media):
            _item_media = reader.tgread_object()
            _list_media.append(_item_media)
        _args['media'] = _list_media
        return cls(**_args)


class EditPreviewMediaRequest(TLRequest):
    """TL type: bots.editPreviewMedia"""
    CONSTRUCTOR_ID = 0x8525606f
    SUBCLASS_OF_ID = 0x562abc2d

    def __init__(self, bot: 'TypeInputUser', lang_code: str, media: 'TypeInputMedia', new_media: 'TypeInputMedia'):
        self.bot = bot
        self.lang_code = lang_code
        self.media = media
        self.new_media = new_media

    def to_dict(self):
        return {
            '_': 'EditPreviewMediaRequest',
            'bot': self.bot,
            'lang_code': self.lang_code,
            'media': self.media,
            'new_media': self.new_media,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        buf.write(bytes(self.media))
        buf.write(bytes(self.new_media))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        _val_media = reader.tgread_object()
        _args['media'] = _val_media
        _val_new_media = reader.tgread_object()
        _args['new_media'] = _val_new_media
        return cls(**_args)


class GetAdminedBotsRequest(TLRequest):
    """TL type: bots.getAdminedBots"""
    CONSTRUCTOR_ID = 0xb0711d83
    SUBCLASS_OF_ID = 0x0406da4d

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetAdminedBotsRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetBotCommandsRequest(TLRequest):
    """TL type: bots.getBotCommands"""
    CONSTRUCTOR_ID = 0xe34c0dd6
    SUBCLASS_OF_ID = 0xfae91529

    def __init__(self, scope: 'TypeBotCommandScope', lang_code: str):
        self.scope = scope
        self.lang_code = lang_code

    def to_dict(self):
        return {
            '_': 'GetBotCommandsRequest',
            'scope': self.scope,
            'lang_code': self.lang_code,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.scope))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_scope = reader.tgread_object()
        _args['scope'] = _val_scope
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        return cls(**_args)


class GetBotInfoRequest(TLRequest):
    """TL type: bots.getBotInfo"""
    CONSTRUCTOR_ID = 0xdcd914fd
    SUBCLASS_OF_ID = 0xca7b2235

    def __init__(self, lang_code: str, bot: Optional['TypeInputUser'] = None):
        self.lang_code = lang_code
        self.bot = bot

    def to_dict(self):
        return {
            '_': 'GetBotInfoRequest',
            'lang_code': self.lang_code,
            'bot': self.bot,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.bot is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        if self.bot is not None:
            buf.write(bytes(self.bot))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_bot = reader.tgread_object()
            _args['bot'] = _val_bot
        else:
            _args['bot'] = None
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        return cls(**_args)


class GetBotMenuButtonRequest(TLRequest):
    """TL type: bots.getBotMenuButton"""
    CONSTRUCTOR_ID = 0x9c60eb28
    SUBCLASS_OF_ID = 0x4c71bd3c

    def __init__(self, user_id: 'TypeInputUser'):
        self.user_id = user_id

    def to_dict(self):
        return {
            '_': 'GetBotMenuButtonRequest',
            'user_id': self.user_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.user_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        return cls(**_args)


class GetBotRecommendationsRequest(TLRequest):
    """TL type: bots.getBotRecommendations"""
    CONSTRUCTOR_ID = 0xa1b70815
    SUBCLASS_OF_ID = 0xf24bf0c0

    def __init__(self, bot: 'TypeInputUser'):
        self.bot = bot

    def to_dict(self):
        return {
            '_': 'GetBotRecommendationsRequest',
            'bot': self.bot,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        return cls(**_args)


class GetPopularAppBotsRequest(TLRequest):
    """TL type: bots.getPopularAppBots"""
    CONSTRUCTOR_ID = 0xc2510192
    SUBCLASS_OF_ID = 0x7b64be7d

    def __init__(self, offset: str, limit: int):
        self.offset = offset
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetPopularAppBotsRequest',
            'offset': self.offset,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_offset = reader.tgread_string()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetPreviewInfoRequest(TLRequest):
    """TL type: bots.getPreviewInfo"""
    CONSTRUCTOR_ID = 0x423ab3ad
    SUBCLASS_OF_ID = 0xf0c27f35

    def __init__(self, bot: 'TypeInputUser', lang_code: str):
        self.bot = bot
        self.lang_code = lang_code

    def to_dict(self):
        return {
            '_': 'GetPreviewInfoRequest',
            'bot': self.bot,
            'lang_code': self.lang_code,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        return cls(**_args)


class GetPreviewMediasRequest(TLRequest):
    """TL type: bots.getPreviewMedias"""
    CONSTRUCTOR_ID = 0xa2a5594d
    SUBCLASS_OF_ID = 0xf9862a78

    def __init__(self, bot: 'TypeInputUser'):
        self.bot = bot

    def to_dict(self):
        return {
            '_': 'GetPreviewMediasRequest',
            'bot': self.bot,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        return cls(**_args)


class InvokeWebViewCustomMethodRequest(TLRequest):
    """TL type: bots.invokeWebViewCustomMethod"""
    CONSTRUCTOR_ID = 0x087fc5e7
    SUBCLASS_OF_ID = 0xad0352e8

    def __init__(self, bot: 'TypeInputUser', custom_method: str, params: 'TypeDataJSON'):
        self.bot = bot
        self.custom_method = custom_method
        self.params = params

    def to_dict(self):
        return {
            '_': 'InvokeWebViewCustomMethodRequest',
            'bot': self.bot,
            'custom_method': self.custom_method,
            'params': self.params,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.custom_method))
        buf.write(bytes(self.params))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_custom_method = reader.tgread_string()
        _args['custom_method'] = _val_custom_method
        _val_params = reader.tgread_object()
        _args['params'] = _val_params
        return cls(**_args)


class ReorderPreviewMediasRequest(TLRequest):
    """TL type: bots.reorderPreviewMedias"""
    CONSTRUCTOR_ID = 0xb627f3aa
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, bot: 'TypeInputUser', lang_code: str, order: List['TypeInputMedia']):
        self.bot = bot
        self.lang_code = lang_code
        self.order = order

    def to_dict(self):
        return {
            '_': 'ReorderPreviewMediasRequest',
            'bot': self.bot,
            'lang_code': self.lang_code,
            'order': self.order,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.order)))
        for item in self.order:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        reader.read_int(signed=False)  # skip vector id
        _count_order = reader.read_int()
        _list_order = []
        for _ in range(_count_order):
            _item_order = reader.tgread_object()
            _list_order.append(_item_order)
        _args['order'] = _list_order
        return cls(**_args)


class ReorderUsernamesRequest(TLRequest):
    """TL type: bots.reorderUsernames"""
    CONSTRUCTOR_ID = 0x9709b1c2
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, bot: 'TypeInputUser', order: List[str]):
        self.bot = bot
        self.order = order

    def to_dict(self):
        return {
            '_': 'ReorderUsernamesRequest',
            'bot': self.bot,
            'order': self.order,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.order)))
        for item in self.order:
            buf.write(TLObject.serialize_bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        reader.read_int(signed=False)  # skip vector id
        _count_order = reader.read_int()
        _list_order = []
        for _ in range(_count_order):
            _item_order = reader.tgread_string()
            _list_order.append(_item_order)
        _args['order'] = _list_order
        return cls(**_args)


class ResetBotCommandsRequest(TLRequest):
    """TL type: bots.resetBotCommands"""
    CONSTRUCTOR_ID = 0x3d8de0f9
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, scope: 'TypeBotCommandScope', lang_code: str):
        self.scope = scope
        self.lang_code = lang_code

    def to_dict(self):
        return {
            '_': 'ResetBotCommandsRequest',
            'scope': self.scope,
            'lang_code': self.lang_code,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.scope))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_scope = reader.tgread_object()
        _args['scope'] = _val_scope
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        return cls(**_args)


class SendCustomRequestRequest(TLRequest):
    """TL type: bots.sendCustomRequest"""
    CONSTRUCTOR_ID = 0xaa2769ed
    SUBCLASS_OF_ID = 0xad0352e8

    def __init__(self, custom_method: str, params: 'TypeDataJSON'):
        self.custom_method = custom_method
        self.params = params

    def to_dict(self):
        return {
            '_': 'SendCustomRequestRequest',
            'custom_method': self.custom_method,
            'params': self.params,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.custom_method))
        buf.write(bytes(self.params))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_custom_method = reader.tgread_string()
        _args['custom_method'] = _val_custom_method
        _val_params = reader.tgread_object()
        _args['params'] = _val_params
        return cls(**_args)


class SetBotBroadcastDefaultAdminRightsRequest(TLRequest):
    """TL type: bots.setBotBroadcastDefaultAdminRights"""
    CONSTRUCTOR_ID = 0x788464e1
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, admin_rights: 'TypeChatAdminRights'):
        self.admin_rights = admin_rights

    def to_dict(self):
        return {
            '_': 'SetBotBroadcastDefaultAdminRightsRequest',
            'admin_rights': self.admin_rights,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.admin_rights))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_admin_rights = reader.tgread_object()
        _args['admin_rights'] = _val_admin_rights
        return cls(**_args)


class SetBotCommandsRequest(TLRequest):
    """TL type: bots.setBotCommands"""
    CONSTRUCTOR_ID = 0x0517165a
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, scope: 'TypeBotCommandScope', lang_code: str, commands: List['TypeBotCommand']):
        self.scope = scope
        self.lang_code = lang_code
        self.commands = commands

    def to_dict(self):
        return {
            '_': 'SetBotCommandsRequest',
            'scope': self.scope,
            'lang_code': self.lang_code,
            'commands': self.commands,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.scope))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.commands)))
        for item in self.commands:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_scope = reader.tgread_object()
        _args['scope'] = _val_scope
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        reader.read_int(signed=False)  # skip vector id
        _count_commands = reader.read_int()
        _list_commands = []
        for _ in range(_count_commands):
            _item_commands = reader.tgread_object()
            _list_commands.append(_item_commands)
        _args['commands'] = _list_commands
        return cls(**_args)


class SetBotGroupDefaultAdminRightsRequest(TLRequest):
    """TL type: bots.setBotGroupDefaultAdminRights"""
    CONSTRUCTOR_ID = 0x925ec9ea
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, admin_rights: 'TypeChatAdminRights'):
        self.admin_rights = admin_rights

    def to_dict(self):
        return {
            '_': 'SetBotGroupDefaultAdminRightsRequest',
            'admin_rights': self.admin_rights,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.admin_rights))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_admin_rights = reader.tgread_object()
        _args['admin_rights'] = _val_admin_rights
        return cls(**_args)


class SetBotInfoRequest(TLRequest):
    """TL type: bots.setBotInfo"""
    CONSTRUCTOR_ID = 0x10cf3123
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, lang_code: str, bot: Optional['TypeInputUser'] = None, name: Optional[str] = None, about: Optional[str] = None, description: Optional[str] = None):
        self.lang_code = lang_code
        self.bot = bot
        self.name = name
        self.about = about
        self.description = description

    def to_dict(self):
        return {
            '_': 'SetBotInfoRequest',
            'lang_code': self.lang_code,
            'bot': self.bot,
            'name': self.name,
            'about': self.about,
            'description': self.description,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.bot is not None:
            flags |= (1 << 2)
        if self.name is not None:
            flags |= (1 << 3)
        if self.about is not None:
            flags |= (1 << 0)
        if self.description is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        if self.bot is not None:
            buf.write(bytes(self.bot))
        if self.name is not None:
            buf.write(TLObject.serialize_bytes(self.name))
        if self.about is not None:
            buf.write(TLObject.serialize_bytes(self.about))
        if self.description is not None:
            buf.write(TLObject.serialize_bytes(self.description))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 2):
            _val_bot = reader.tgread_object()
            _args['bot'] = _val_bot
        else:
            _args['bot'] = None
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        if flags & (1 << 3):
            _val_name = reader.tgread_string()
            _args['name'] = _val_name
        else:
            _args['name'] = None
        if flags & (1 << 0):
            _val_about = reader.tgread_string()
            _args['about'] = _val_about
        else:
            _args['about'] = None
        if flags & (1 << 1):
            _val_description = reader.tgread_string()
            _args['description'] = _val_description
        else:
            _args['description'] = None
        return cls(**_args)


class SetBotMenuButtonRequest(TLRequest):
    """TL type: bots.setBotMenuButton"""
    CONSTRUCTOR_ID = 0x4504d54f
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, user_id: 'TypeInputUser', button: 'TypeBotMenuButton'):
        self.user_id = user_id
        self.button = button

    def to_dict(self):
        return {
            '_': 'SetBotMenuButtonRequest',
            'user_id': self.user_id,
            'button': self.button,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.user_id))
        buf.write(bytes(self.button))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_button = reader.tgread_object()
        _args['button'] = _val_button
        return cls(**_args)


class SetCustomVerificationRequest(TLRequest):
    """TL type: bots.setCustomVerification"""
    CONSTRUCTOR_ID = 0x8b89dfbd
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPeer', enabled: Optional[bool] = None, bot: Optional['TypeInputUser'] = None, custom_description: Optional[str] = None):
        self.peer = peer
        self.enabled = enabled
        self.bot = bot
        self.custom_description = custom_description

    def to_dict(self):
        return {
            '_': 'SetCustomVerificationRequest',
            'peer': self.peer,
            'enabled': self.enabled,
            'bot': self.bot,
            'custom_description': self.custom_description,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.enabled:
            flags |= (1 << 1)
        if self.bot is not None:
            flags |= (1 << 0)
        if self.custom_description is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        if self.bot is not None:
            buf.write(bytes(self.bot))
        if self.custom_description is not None:
            buf.write(TLObject.serialize_bytes(self.custom_description))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['enabled'] = bool(flags & (1 << 1))
        if flags & (1 << 0):
            _val_bot = reader.tgread_object()
            _args['bot'] = _val_bot
        else:
            _args['bot'] = None
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        if flags & (1 << 2):
            _val_custom_description = reader.tgread_string()
            _args['custom_description'] = _val_custom_description
        else:
            _args['custom_description'] = None
        return cls(**_args)


class ToggleUserEmojiStatusPermissionRequest(TLRequest):
    """TL type: bots.toggleUserEmojiStatusPermission"""
    CONSTRUCTOR_ID = 0x06de6392
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, bot: 'TypeInputUser', enabled: bool):
        self.bot = bot
        self.enabled = enabled

    def to_dict(self):
        return {
            '_': 'ToggleUserEmojiStatusPermissionRequest',
            'bot': self.bot,
            'enabled': self.enabled,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(struct.pack('<I', 0x997275b5 if self.enabled else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_enabled = reader.tgread_bool()
        _args['enabled'] = _val_enabled
        return cls(**_args)


class ToggleUsernameRequest(TLRequest):
    """TL type: bots.toggleUsername"""
    CONSTRUCTOR_ID = 0x053ca973
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, bot: 'TypeInputUser', username: str, active: bool):
        self.bot = bot
        self.username = username
        self.active = active

    def to_dict(self):
        return {
            '_': 'ToggleUsernameRequest',
            'bot': self.bot,
            'username': self.username,
            'active': self.active,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.bot))
        buf.write(TLObject.serialize_bytes(self.username))
        buf.write(struct.pack('<I', 0x997275b5 if self.active else 0xbc799737))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_username = reader.tgread_string()
        _args['username'] = _val_username
        _val_active = reader.tgread_bool()
        _args['active'] = _val_active
        return cls(**_args)


class UpdateStarRefProgramRequest(TLRequest):
    """TL type: bots.updateStarRefProgram"""
    CONSTRUCTOR_ID = 0x778b5ab3
    SUBCLASS_OF_ID = 0x9888e002

    def __init__(self, bot: 'TypeInputUser', commission_permille: int, duration_months: Optional[int] = None):
        self.bot = bot
        self.commission_permille = commission_permille
        self.duration_months = duration_months

    def to_dict(self):
        return {
            '_': 'UpdateStarRefProgramRequest',
            'bot': self.bot,
            'commission_permille': self.commission_permille,
            'duration_months': self.duration_months,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.duration_months is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.bot))
        buf.write(struct.pack('<i', self.commission_permille))
        if self.duration_months is not None:
            buf.write(struct.pack('<i', self.duration_months))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_bot = reader.tgread_object()
        _args['bot'] = _val_bot
        _val_commission_permille = reader.read_int()
        _args['commission_permille'] = _val_commission_permille
        if flags & (1 << 0):
            _val_duration_months = reader.read_int()
            _args['duration_months'] = _val_duration_months
        else:
            _args['duration_months'] = None
        return cls(**_args)


class UpdateUserEmojiStatusRequest(TLRequest):
    """TL type: bots.updateUserEmojiStatus"""
    CONSTRUCTOR_ID = 0xed9f30c5
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, user_id: 'TypeInputUser', emoji_status: 'TypeEmojiStatus'):
        self.user_id = user_id
        self.emoji_status = emoji_status

    def to_dict(self):
        return {
            '_': 'UpdateUserEmojiStatusRequest',
            'user_id': self.user_id,
            'emoji_status': self.emoji_status,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.user_id))
        buf.write(bytes(self.emoji_status))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_emoji_status = reader.tgread_object()
        _args['emoji_status'] = _val_emoji_status
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class Albums(TLObject):
    """TL type: stories.albums"""
    CONSTRUCTOR_ID = 0xc3987a3a
    SUBCLASS_OF_ID = 0x05a73d39

    def __init__(self, hash: int, albums: List['TypeStoryAlbum']):
        self.hash = hash
        self.albums = albums

    def to_dict(self):
        return {
            '_': 'Albums',
            'hash': self.hash,
            'albums': self.albums,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.albums)))
        for item in self.albums:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_albums = reader.read_int()
        _list_albums = []
        for _ in range(_count_albums):
            _item_albums = reader.tgread_object()
            _list_albums.append(_item_albums)
        _args['albums'] = _list_albums
        return cls(**_args)


class AlbumsNotModified(TLObject):
    """TL type: stories.albumsNotModified"""
    CONSTRUCTOR_ID = 0x564edaeb
    SUBCLASS_OF_ID = 0x05a73d39

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'AlbumsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class AllStories(TLObject):
    """TL type: stories.allStories"""
    CONSTRUCTOR_ID = 0x6efc5e81
    SUBCLASS_OF_ID = 0x7e60d0cd

    def __init__(self, count: int, state: str, peer_stories: List['TypePeerStories'], chats: List['TypeChat'], users: List['TypeUser'], stealth_mode: 'TypeStoriesStealthMode', has_more: Optional[bool] = None):
        self.count = count
        self.state = state
        self.peer_stories = peer_stories
        self.chats = chats
        self.users = users
        self.stealth_mode = stealth_mode
        self.has_more = has_more

    def to_dict(self):
        return {
            '_': 'AllStories',
            'count': self.count,
            'state': self.state,
            'peer_stories': self.peer_stories,
            'chats': self.chats,
            'users': self.users,
            'stealth_mode': self.stealth_mode,
            'has_more': self.has_more,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.has_more:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(TLObject.serialize_bytes(self.state))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.peer_stories)))
        for item in self.peer_stories:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        buf.write(bytes(self.stealth_mode))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['has_more'] = bool(flags & (1 << 0))
        _val_count = reader.read_int()
        _args['count'] = _val_count
        _val_state = reader.tgread_string()
        _args['state'] = _val_state
        reader.read_int(signed=False)  # skip vector id
        _count_peer_stories = reader.read_int()
        _list_peer_stories = []
        for _ in range(_count_peer_stories):
            _item_peer_stories = reader.tgread_object()
            _list_peer_stories.append(_item_peer_stories)
        _args['peer_stories'] = _list_peer_stories
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        _val_stealth_mode = reader.tgread_object()
        _args['stealth_mode'] = _val_stealth_mode
        return cls(**_args)


class AllStoriesNotModified(TLObject):
    """TL type: stories.allStoriesNotModified"""
    CONSTRUCTOR_ID = 0x1158fe3e
    SUBCLASS_OF_ID = 0x7e60d0cd

    def __init__(self, state: str, stealth_mode: 'TypeStoriesStealthMode'):
        self.state = state
        self.stealth_mode = stealth_mode

    def to_dict(self):
        return {
            '_': 'AllStoriesNotModified',
            'state': self.state,
            'stealth_mode': self.stealth_mode,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.state))
        buf.write(bytes(self.stealth_mode))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_state = reader.tgread_string()
        _args['state'] = _val_state
        _val_stealth_mode = reader.tgread_object()
        _args['stealth_mode'] = _val_stealth_mode
        return cls(**_args)


class CanSendStoryCount(TLObject):
    """TL type: stories.canSendStoryCount"""
    CONSTRUCTOR_ID = 0xc387c04e
    SUBCLASS_OF_ID = 0xcb53a298

    def __init__(self, count_remains: int):
        self.count_remains = count_remains

    def to_dict(self):
        return {
            '_': 'CanSendStoryCount',
            'count_remains': self.count_remains,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count_remains))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count_remains = reader.read_int()
        _args['count_remains'] = _val_count_remains
        return cls(**_args)


class FoundStories(TLObject):
    """TL type: stories.foundStories"""
    CONSTRUCTOR_ID = 0xe2de7737
    SUBCLASS_OF_ID = 0x17790b35

    def __init__(self, count: int, stories: List['TypeFoundStory'], chats: List['TypeChat'], users: List['TypeUser'], next_offset: Optional[str] = None):
        self.count = count
        self.stories = stories
        self.chats = chats
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'FoundStories',
            'count': self.count,
            'stories': self.stories,
            'chats': self.chats,
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.stories)))
        for item in self.stories:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_stories = reader.read_int()
        _list_stories = []
        for _ in range(_count_stories):
            _item_stories = reader.tgread_object()
            _list_stories.append(_item_stories)
        _args['stories'] = _list_stories
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PeerStories(TLObject):
    """TL type: stories.peerStories"""
    CONSTRUCTOR_ID = 0xcae68768
    SUBCLASS_OF_ID = 0x9d56cfd0

    def __init__(self, stories: 'TypePeerStories', chats: List['TypeChat'], users: List['TypeUser']):
        self.stories = stories
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'PeerStories',
            'stories': self.stories,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.stories))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_stories = reader.tgread_object()
        _args['stories'] = _val_stories
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class Stories(TLObject):
    """TL type: stories.stories"""
    CONSTRUCTOR_ID = 0x63c3dd0a
    SUBCLASS_OF_ID = 0x251c0c2c

    def __init__(self, count: int, stories: List['TypeStoryItem'], chats: List['TypeChat'], users: List['TypeUser'], pinned_to_top: Optional[List[int]] = None):
        self.count = count
        self.stories = stories
        self.chats = chats
        self.users = users
        self.pinned_to_top = pinned_to_top

    def to_dict(self):
        return {
            '_': 'Stories',
            'count': self.count,
            'stories': self.stories,
            'chats': self.chats,
            'users': self.users,
            'pinned_to_top': self.pinned_to_top,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.pinned_to_top is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.stories)))
        for item in self.stories:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.pinned_to_top is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.pinned_to_top)))
            for item in self.pinned_to_top:
                buf.write(struct.pack('<i', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_stories = reader.read_int()
        _list_stories = []
        for _ in range(_count_stories):
            _item_stories = reader.tgread_object()
            _list_stories.append(_item_stories)
        _args['stories'] = _list_stories
        if flags & (1 << 0):
            reader.read_int(signed=False)  # skip vector id
            _count_pinned_to_top = reader.read_int()
            _list_pinned_to_top = []
            for _ in range(_count_pinned_to_top):
                _item_pinned_to_top = reader.read_int()
                _list_pinned_to_top.append(_item_pinned_to_top)
            _args['pinned_to_top'] = _list_pinned_to_top
        else:
            _args['pinned_to_top'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class StoryReactionsList(TLObject):
    """TL type: stories.storyReactionsList"""
    CONSTRUCTOR_ID = 0xaa5f789c
    SUBCLASS_OF_ID = 0x046f91e3

    def __init__(self, count: int, reactions: List['TypeStoryReaction'], chats: List['TypeChat'], users: List['TypeUser'], next_offset: Optional[str] = None):
        self.count = count
        self.reactions = reactions
        self.chats = chats
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'StoryReactionsList',
            'count': self.count,
            'reactions': self.reactions,
            'chats': self.chats,
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.reactions)))
        for item in self.reactions:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_reactions = reader.read_int()
        _list_reactions = []
        for _ in range(_count_reactions):
            _item_reactions = reader.tgread_object()
            _list_reactions.append(_item_reactions)
        _args['reactions'] = _list_reactions
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        return cls(**_args)


class StoryViews(TLObject):
    """TL type: stories.storyViews"""
    CONSTRUCTOR_ID = 0xde9eed1d
    SUBCLASS_OF_ID = 0x4b3fc4ba

    def __init__(self, views: List['TypeStoryViews'], users: List['TypeUser']):
        self.views = views
        self.users = users

    def to_dict(self):
        return {
            '_': 'StoryViews',
            'views': self.views,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.views)))
        for item in self.views:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_views = reader.read_int()
        _list_views = []
        for _ in range(_count_views):
            _item_views = reader.tgread_object()
            _list_views.append(_item_views)
        _args['views'] = _list_views
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class StoryViewsList(TLObject):
    """TL type: stories.storyViewsList"""
    CONSTRUCTOR_ID = 0x59d78fc5
    SUBCLASS_OF_ID = 0xb9437560

    def __init__(self, count: int, views_count: int, forwards_count: int, reactions_count: int, views: List['TypeStoryView'], chats: List['TypeChat'], users: List['TypeUser'], next_offset: Optional[str] = None):
        self.count = count
        self.views_count = views_count
        self.forwards_count = forwards_count
        self.reactions_count = reactions_count
        self.views = views
        self.chats = chats
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'StoryViewsList',
            'count': self.count,
            'views_count': self.views_count,
            'forwards_count': self.forwards_count,
            'reactions_count': self.reactions_count,
            'views': self.views,
            'chats': self.chats,
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', self.views_count))
        buf.write(struct.pack('<i', self.forwards_count))
        buf.write(struct.pack('<i', self.reactions_count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.views)))
        for item in self.views:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        _val_views_count = reader.read_int()
        _args['views_count'] = _val_views_count
        _val_forwards_count = reader.read_int()
        _args['forwards_count'] = _val_forwards_count
        _val_reactions_count = reader.read_int()
        _args['reactions_count'] = _val_reactions_count
        reader.read_int(signed=False)  # skip vector id
        _count_views = reader.read_int()
        _list_views = []
        for _ in range(_count_views):
            _item_views = reader.tgread_object()
            _list_views.append(_item_views)
        _args['views'] = _list_views
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class GetFullUserRequest(TLRequest):
    """TL type: users.getFullUser"""
    CONSTRUCTOR_ID = 0xb60f5918
    SUBCLASS_OF_ID = 0x83df9df5

    def __init__(self, id: 'TypeInputUser'):
        self.id = id

    def to_dict(self):
        return {
            '_': 'GetFullUserRequest',
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        return cls(**_args)


class GetRequirementsToContactRequest(TLRequest):
    """TL type: users.getRequirementsToContact"""
    CONSTRUCTOR_ID = 0xd89a83a3
    SUBCLASS_OF_ID = 0x322623c3

    def __init__(self, id: List['TypeInputUser']):
        self.id = id

    def to_dict(self):
        return {
            '_': 'GetRequirementsToContactRequest',
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.tgread_object()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class GetSavedMusicRequest(TLRequest):
    """TL type: users.getSavedMusic"""
    CONSTRUCTOR_ID = 0x788d7fe3
    SUBCLASS_OF_ID = 0xf813ae37

    def __init__(self, id: 'TypeInputUser', offset: int, limit: int, hash: int):
        self.id = id
        self.offset = offset
        self.limit = limit
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetSavedMusicRequest',
            'id': self.id,
            'offset': self.offset,
            'limit': self.limit,
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.id))
        buf.write(struct.pack('<i', self.offset))
        buf.write(struct.pack('<i', self.limit))
        buf.write(struct.pack('<q', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        _val_offset = reader.read_int()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetSavedMusicByIdRequest(TLRequest):
    """TL type: users.getSavedMusicByID"""
    CONSTRUCTOR_ID = 0x7573a4e9
    SUBCLASS_OF_ID = 0xf813ae37

    def __init__(self, id: 'TypeInputUser', documents: List['TypeInputDocument']):
        self.id = id
        self.documents = documents

    def to_dict(self):
        return {
            '_': 'GetSavedMusicByIdRequest',
            'id': self.id,
            'documents': self.documents,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.id))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.documents)))
        for item in self.documents:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        reader.read_int(signed=False)  # skip vector id
        _count_documents = reader.read_int()
        _list_documents = []
        for _ in range(_count_documents):
            _item_documents = reader.tgread_object()
            _list_documents.append(_item_documents)
        _args['documents'] = _list_documents
        return cls(**_args)


class GetUsersRequest(TLRequest):
    """TL type: users.getUsers"""
    CONSTRUCTOR_ID = 0x0d91a548
    SUBCLASS_OF_ID = 0x0406da4d

    def __init__(self, id: List['TypeInputUser']):
        self.id = id

    def to_dict(self):
        return {
            '_': 'GetUsersRequest',
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.id)))
        for item in self.id:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_id = reader.read_int()
        _list_id = []
        for _ in range(_count_id):
            _item_id = reader.tgread_object()
            _list_id.append(_item_id)
        _args['id'] = _list_id
        return cls(**_args)


class SetSecureValueErrorsRequest(TLRequest):
    """TL type: users.setSecureValueErrors"""
    CONSTRUCTOR_ID = 0x90c894b5
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, id: 'TypeInputUser', errors: List['TypeSecureValueError']):
        self.id = id
        self.errors = errors

    def to_dict(self):
        return {
            '_': 'SetSecureValueErrorsRequest',
            'id': self.id,
            'errors': self.errors,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.id))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.errors)))
        for item in self.errors:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        reader.read_int(signed=False)  # skip vector id
        _count_errors = reader.read_int()
        _list_errors = []
        for _ in range(_count_errors):
            _item_errors = reader.tgread_object()
            _list_errors.append(_item_errors)
        _args['errors'] = _list_errors
        return cls(**_args)


class SuggestBirthdayRequest(TLRequest):
    """TL type: users.suggestBirthday"""
    CONSTRUCTOR_ID = 0xfc533372
    SUBCLASS_OF_ID = 0x8af52aac

    def __init__(self, id: 'TypeInputUser', birthday: 'TypeBirthday'):
        self.id = id
        self.birthday = birthday

    def to_dict(self):
        return {
            '_': 'SuggestBirthdayRequest',
            'id': self.id,
            'birthday': self.birthday,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.id))
        buf.write(bytes(self.birthday))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        _val_birthday = reader.tgread_object()
        _args['birthday'] = _val_birthday
        return cls(**_args)




"""
2FA (Two-Factor Authentication) SRP-2048 implementation for Telegram.
Verified against Telethon's reference implementation.
https://core.telegram.org/api/srp
"""
import hashlib
import os

SIZE_FOR_HASH = 256  # all big-number byte buffers hashed must be this size


# ── Low-level helpers ──────────────────────────────────────────────────────────

def _sha256(*parts: bytes) -> bytes:
    h = hashlib.sha256()
    for p in parts:
        h.update(p)
    return h.digest()


def _pad256(b: bytes) -> bytes:
    """Left-pad bytes to exactly 256 bytes (Telegram always hashes 256-byte values)."""
    return b.rjust(SIZE_FOR_HASH, b'\x00')


def _int_to_256(n: int) -> bytes:
    return n.to_bytes(SIZE_FOR_HASH, 'big')


def _xor(a: bytes, b: bytes) -> bytes:
    return bytes(x ^ y for x, y in zip(a, b))


# ── Password hash  (matches Telethon's compute_hash exactly) ──────────────────
#
# Telegram's formula (from TDesktop source / Telethon):
#
#   SH(data, salt) = SHA256(salt || data || salt)
#
#   hash1 = SH(pw, salt1)
#   hash2 = SH(hash1, salt2)          ← double-hash before PBKDF2
#   hash3 = PBKDF2-SHA512(hash2, salt1, 100000)   ← salt = salt1
#   x     = SH(hash3, salt2)
#
# This is intentionally different from the simplified PH1/PH2 pseudocode
# sometimes quoted online — Telethon's formula is the one that actually works.

def _compute_password_hash(salt1: bytes, salt2: bytes, password: bytes) -> bytes:
    hash1 = _sha256(salt1, password, salt1)          # SH(pw, salt1)
    hash2 = _sha256(salt2, hash1, salt2)             # SH(hash1, salt2)
    hash3 = hashlib.pbkdf2_hmac('sha512', hash2, salt1, 100000)
    return _sha256(salt2, hash3, salt2)              # SH(hash3, salt2)


# ── Mod-exp range checks (prevents MITM / bad-server attacks) ─────────────────

def _is_good_mod_exp(value: int, prime: int) -> bool:
    """
    Validates that a mod-exp result is safe (at least 2048-64 = 1984 bits,
    and prime - value also has at least 1984 bits).
    """
    diff = prime - value
    min_bits = 2048 - 64
    max_bytes = 256
    if (diff < 0 or
            diff.bit_length() < min_bits or
            value.bit_length() < min_bits or
            (value.bit_length() + 7) // 8 > max_bytes):
        return False
    return True


# ── Main entry point ───────────────────────────────────────────────────────────

async def compute_srp_answer(password_obj, password: str):
    """
    Compute InputCheckPasswordSRP for auth.checkPassword.
    Raises ValueError if the password object is malformed.
    The server returns PASSWORD_HASH_INVALID if the password string is wrong.
    """
    from .tl.types import InputCheckPasswordSrp as InputCheckPasswordSRP

    if not password_obj.has_password:
        raise ValueError('This account does not have a 2FA password set.')

    algo = password_obj.current_algo
    if algo is None or not hasattr(algo, 'salt1'):
        raise ValueError(
            'Server returned an unsupported KDF algorithm: '
            + type(algo).__name__
        )

    salt1    = algo.salt1
    salt2    = algo.salt2
    g_int    = algo.g
    p_bytes  = _pad256(algo.p)            # always hash with 256-byte p
    p_int    = int.from_bytes(p_bytes, 'big')
    srp_id   = password_obj.srp_id
    g_bytes  = _int_to_256(g_int)        # g padded to 256 bytes

    # srp_B from Telegram may have leading zeros stripped — restore them
    b_bytes  = _pad256(password_obj.srp_B)
    B        = int.from_bytes(b_bytes, 'big')

    if not (0 < B < p_int):
        raise ValueError('Server sent an invalid g_b (out of range).')

    # ── Step 1: compute x (the password-derived secret) ───────────────────────
    pw_bytes = password.encode('utf-8')
    x_bytes  = _compute_password_hash(salt1, salt2, pw_bytes)
    x        = int.from_bytes(x_bytes, 'big')

    # ── Step 2: k = H(p_256 | g_256) ──────────────────────────────────────────
    k = int.from_bytes(_sha256(p_bytes, g_bytes), 'big')

    # ── Step 3: generate A = g^a mod p, retrying until A is safe ─────────────
    while True:
        a_bytes = os.urandom(256)
        a       = int.from_bytes(a_bytes, 'big')
        A       = pow(g_int, a, p_int)
        if _is_good_mod_exp(A, p_int):
            break
    a_for_hash = _int_to_256(A)

    # ── Step 4: u = H(A_256 | B_256) ──────────────────────────────────────────
    u = int.from_bytes(_sha256(a_for_hash, b_bytes), 'big')
    if u == 0:
        raise ValueError('u == 0; retry.')

    # ── Step 5: g_b = (B - k*g^x) mod p, check it's safe ────────────────────
    g_x  = pow(g_int, x, p_int)
    kg_x = (k * g_x) % p_int
    g_b  = (B - kg_x) % p_int

    if not _is_good_mod_exp(g_b, p_int):
        raise ValueError('Computed g_b failed range check; possible MITM.')

    # ── Step 6: session key K = H(g_b^(a + u*x) mod p) ───────────────────────
    S       = pow(g_b, a + u * x, p_int)
    K_bytes = _sha256(_int_to_256(S))

    # ── Step 7: M1 = H(H(p) XOR H(g) | H(s1) | H(s2) | A | B | K) ──────────
    M1 = _sha256(
        _xor(_sha256(p_bytes), _sha256(g_bytes)),
        _sha256(salt1),
        _sha256(salt2),
        a_for_hash,
        b_bytes,
        K_bytes,
    )

    return InputCheckPasswordSRP(
        srp_id=srp_id,
        A=a_for_hash,
        M1=M1,
    )
